﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Syncfusion.Windows.Forms.Diagram;
using System.Drawing.Imaging;
using Syncfusion.Windows.Forms.Tools;
using Syncfusion.Windows.Forms.Diagram.Controls;
using System.Drawing.Drawing2D;
using Syncfusion.Windows.Forms;
using System.IO;
using System.Drawing.Printing;
using Syncfusion.SVG.IO;
using System.Collections;

namespace DifferentLayoutInEveryLevel
{
    public partial class Form1 : Form
    {

        #region Form Initialize
        public Form1()
        {
            InitializeComponent();
            this.diagram2.Model.BoundaryConstraintsEnabled = true;
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            diagram2.BeginUpdate();
            NodeCollection radialnodes = new NodeCollection();
            NodeCollection directed1 = new NodeCollection();
            NodeCollection directed2 = new NodeCollection();
            NodeCollection directed3 = new NodeCollection();
            PopulateFields(SetDataSource(), ref radialnodes, ref directed1, ref directed2, ref directed3);

            //Applying different layout for different levels
            RadialLayouting(directed1);
            DirectedLayouting(directed2);
            DirectedLayouting(directed3);


          
            diagram2.EndUpdate();
            diagram2.View.SelectionList.Clear();
        }
        private void PopulateFields(DataTable dt, ref NodeCollection radialnodes, ref NodeCollection directed1, ref NodeCollection directed2, ref NodeCollection directed3)
        {
            for (int i = 0; i < dt.Rows.Count; i++)
            {
                Syncfusion.Windows.Forms.Diagram.RoundRect rect = new Syncfusion.Windows.Forms.Diagram.RoundRect(0F, 0F, 120F, 60F, MeasureUnits.Pixel);
                rect.FillStyle.Color = Color.FromArgb(255, 86, 4);
                rect.FillStyle.ForeColor = Color.FromArgb(255, 165, 74);
                rect.LineStyle.LineColor = Color.White;
                rect.FillStyle.Type = FillStyleType.LinearGradient;
                //rect.EnableShading = true;
                rect.Name = dt.Rows[i]["Name"].ToString();
                Syncfusion.Windows.Forms.Diagram.Label label = new Syncfusion.Windows.Forms.Diagram.Label(rect, dt.Rows[i]["Name"].ToString());
                label.FontStyle.Family = "Segoe UI";
                label.FontStyle.Size = 9F;
                label.FontColorStyle.Color = Color.White;
                rect.Labels.Add(label);
                Syncfusion.Windows.Forms.Diagram.Label label1 = new Syncfusion.Windows.Forms.Diagram.Label(rect, dt.Rows[i]["Designation"].ToString());
                label1.FontStyle.Family = "Segoe UI";
                label1.FontStyle.Size = 9F;
                label1.FontColorStyle.Color = Color.White;
                label1.OffsetX = rect.BoundingRectangle.Width / 2 - label1.Size.Width / 2;
                label1.OffsetY = rect.BoundingRectangle.Height / 2 + 5;
                rect.Labels.Add(label1);
                diagram2.Model.AppendChild(rect);
                AddNodesforLayout(rect, ref radialnodes);
                if (!(string.IsNullOrEmpty(dt.Rows[i]["ParentName"].ToString())))
                {
                    Syncfusion.Windows.Forms.Diagram.RoundRect parentNode = diagram2.Model.Nodes.FindNodeByName(dt.Rows[i]["ParentName"].ToString()) as Syncfusion.Windows.Forms.Diagram.RoundRect;
                    string ss = parentNode.Name;
                    ConnectNodes(parentNode, rect, ref radialnodes);
                }
                if (rect.Name == "John Smith" || rect.Name == "Joe Robert" || rect.Name == "Jack Kent")
                    AddNodesforLayout(rect, ref directed1);
                else if (rect.Name == "Sue Raymond" || rect.Name == "Ron Jones" || rect.Name == "Dave Mason" || rect.Name == "Avinash")
                    AddNodesforLayout(rect, ref directed2);
                else if (rect.Name == "Lisa Simpson" || rect.Name == "Manoj" || rect.Name == "Alivelu" || rect.Name == "Anuradha")
                    AddNodesforLayout(rect, ref directed3);

            }
        }

        private void AddNodesforLayout(RoundRect rect, ref NodeCollection nodes)
        {
            nodes.Add(rect);
            if (rect.CentralPort.Connections.Count > 0)
            {
                foreach (EndPoint endpt in rect.CentralPort.Connections)
                {
                    ConnectorBase conn = endpt.Container as ConnectorBase;
                    nodes.Add(conn);
                }
            }
        }

        #endregion

        #region Private Methods
        /// <summary>
        /// Change's the appearance of Diagram
        /// </summary>
        private void DiagramAppearance()
        {
            this.diagram2.Model.RenderingStyle.SmoothingMode = SmoothingMode.HighQuality;
            this.diagram2.View.BackgroundColor = Color.White;
            this.diagram2.Model.BoundaryConstraintsEnabled = false;
            this.diagram2.View.HandleRenderer.HandleColor = Color.AliceBlue;
            this.diagram2.View.HandleRenderer.HandleOutlineColor = Color.SkyBlue;
            this.diagram2.View.SelectionList.Clear();
        }
        #endregion

        #region InitializeDiagram
        /// <summary>
        /// Generate the Data source for diagram
        /// </summary>
        /// <returns></returns>
        private DataTable SetDataSource()
        {
            DataTable table = new DataTable();
            table.Columns.Add("ID", typeof(string));
            table.Columns.Add("Name", typeof(string));
            table.Columns.Add("Designation", typeof(string));
            table.Columns.Add("ParentName", typeof(string));
            //
            // Here we add five DataRows.
            //
            table.Rows.Add("1", "John Smith", "CEO", string.Empty);
            table.Rows.Add("2", "Joe Robert", "Manager-SLS", "John Smith");
            table.Rows.Add("3", "Jack Kent", "Manager-Mkt", "Joe Robert");
            table.Rows.Add("4", "Ravi kumar", "Manager-DEV", "Jack Kent");

            table.Rows.Add("5", "Sue Raymond", "CSR", "Ravi kumar");
            table.Rows.Add("6", "Lisa Simpson", "CSR", "Ravi kumar");
            table.Rows.Add("7", "Bob Woley", "PM", "Ravi kumar");

            table.Rows.Add("8", "Ron Jones", "Engineer", "Sue Raymond");
            table.Rows.Add("9", "Dave Mason", "Engineer", "Ron Jones");
            table.Rows.Add("10", "Avinash", "CEO", "Dave Mason");


            table.Rows.Add("11", "Manoj", "Manager-SLS", "Lisa Simpson");
            table.Rows.Add("12", "Alivelu", "Manager-Mkt", "Manoj");
            table.Rows.Add("13", "Anuradha", "Manager-DEV", "Alivelu");

            return table;
        }

        /// <summary>
        /// Connects the given nodes
        /// </summary>
        /// <param name="parentNode">Parent Node</param>
        /// <param name="childNode">Child node</param>
        private void ConnectNodes(Node parentNode, Node childNode, ref NodeCollection nodes)
        {
            if (parentNode != null && childNode != null)
            {
                OrgLineConnector conn = new OrgLineConnector(PointF.Empty, new PointF(0F, 1F));
                conn.HeadDecorator.DecoratorShape = DecoratorShape.Filled60Arrow;
                conn.TailDecorator.DecoratorShape = DecoratorShape.Filled60Arrow;
                conn.VerticalDistance = 35F;
                conn.LineStyle.LineColor = Color.Black;
                conn.LineStyle.DashStyle = DashStyle.Dash;
                Decorator decor = conn.HeadDecorator;
                decor.DecoratorShape = DecoratorShape.Filled60Arrow;
                decor.FillStyle.Color = Color.White;
                decor.LineStyle.LineColor = Color.Black;

                this.diagram2.Model.AppendChild(conn);
                nodes.Add(conn);

                parentNode.CentralPort.TryConnect(conn.TailEndPoint);
                childNode.CentralPort.TryConnect(conn.HeadEndPoint);

                this.diagram2.Model.SendToBack(conn);

            }
        }

        #endregion

        #region Helper Methods

        /// <summary>
        /// Create the new member with specified name and title.
        /// </summary>
        /// <param name="strMemberName">The member name.</param>
        /// <param name="strTitle">The title.</param>
        private Node CreateMember(string strMemberName, string strTitle)
        {
            SizeF szSize = new SizeF(140F, 60F);
            Syncfusion.Windows.Forms.Diagram.RoundRect rnRect = new Syncfusion.Windows.Forms.Diagram.RoundRect(0F, 0F, 120F, 60F, MeasureUnits.Pixel);
            rnRect.FillStyle.Color = Color.FromArgb(255, 86, 4);
            rnRect.FillStyle.ForeColor = Color.FromArgb(255, 165, 74);
            rnRect.LineStyle.LineColor = Color.White;
            rnRect.FillStyle.Type = FillStyleType.LinearGradient;
            Syncfusion.Windows.Forms.Diagram.Label label = new Syncfusion.Windows.Forms.Diagram.Label(rnRect, strMemberName);
            label.FontStyle.Family = "Segoe UI";
            label.FontStyle.Size = 9F;
            label.FontColorStyle.Color = Color.White;
            rnRect.Labels.Add(label);
            Syncfusion.Windows.Forms.Diagram.Label label1 = new Syncfusion.Windows.Forms.Diagram.Label(rnRect, strTitle);
            label1.FontStyle.Family = "Segoe UI";
            label1.FontStyle.Size = 9F;
            label1.FontColorStyle.Color = Color.White;
            label1.OffsetX = rnRect.BoundingRectangle.Width / 2 - label1.Size.Width / 2;
            label1.OffsetY = rnRect.BoundingRectangle.Height / 2 + 5;
            rnRect.Labels.Add(label1);
            return rnRect;
        }

        private void RadialLayouting(NodeCollection nodes)
        {
            RadialTreeLayoutManager radialLayout = new RadialTreeLayoutManager();
            radialLayout.AutoLayout = true;
            radialLayout.HorizontalSpacing = 50;
            radialLayout.VerticalSpacing = 50;
            radialLayout.Model = diagram2.Model;
            radialLayout.Nodes.Clear();
            radialLayout.Nodes.AddRange(nodes);
            this.diagram2.LayoutManager = radialLayout;
            this.diagram2.LayoutManager.UpdateLayout(nodes);
            diagram2.Controller.View.SelectionList.Clear();
            diagram2.Controller.View.SelectionList.AddRange(radialLayout.Nodes);
            this.diagram2.UpdateView();
        }

        private void DirectedLayouting(NodeCollection nodes)
        {
            DirectedTreeLayoutManager directLaout = new DirectedTreeLayoutManager();
            directLaout.AutoLayout = true;
            directLaout.HorizontalSpacing = 40;
            directLaout.VerticalSpacing = 40;
            directLaout.Model = diagram2.Model;
            directLaout.Nodes.Clear();
            directLaout.Nodes.AddRange(nodes);
            this.diagram2.LayoutManager = directLaout;
            this.diagram2.LayoutManager.UpdateLayout(nodes);
            diagram2.Controller.View.SelectionList.Clear();
            diagram2.Controller.View.SelectionList.AddRange(directLaout.Nodes);
            this.diagram2.UpdateView();
        }
        #endregion

    }

}